@@ -497,11 +497,11 @@ math.log10                  =
 '返回 `x` 的以10为底的对数。'
 math.max                    =
 '返回参数中最大的值， 大小由 Lua 操作 `<` 决定。'
-math.maxinteger             =
+math.maxinteger['>5.3']     =
 '最大值的整数。'
 math.min                    =
 '返回参数中最小的值， 大小由 Lua 操作 `<` 决定。'
-math.mininteger             =
+math.mininteger['>5.3']     =
 '最小值的整数。'
 math.modf                   =
 '返回 `x` 的整数部分和小数部分。'
@@ -535,11 +535,11 @@ math.tan                    =
 '返回 `x` 的正切值（假定参数是弧度）。'
 math.tanh                   =
 '返回 `x` 的双曲正切值（假定参数是弧度）。'
-math.tointeger              =
+math.tointeger['>5.3']      =
 '如果 `x` 可以转换为一个整数， 返回该整数。'
-math.type                   =
+math.type['>5.3']           =
 '如果 `x` 是整数，返回 `"integer"`， 如果它是浮点数，返回 `"float"`， 如果 `x` 不是数字，返回 `nil`。'
-math.ult                    =
+math.ult['>5.3']            =
 '如果整数 `m` 和 `n` 以无符号整数形式比较， `m` 在 `n` 之下，返回布尔真否则返回假。'
 
 os                          =
@@ -654,10 +654,10 @@ string.pack                 =
 '返回一个打包了（即以二进制形式序列化） v1, v2 等值的二进制字符串。 字符串 fmt 为打包格式（参见 §6.4.2）。'
 string.packsize             =
 [[返回以指定格式用 $string.pack 打包的字符串的长度。 格式化字符串中不可以有变长选项 's' 或 'z' （参见 §6.4.2）。]]
-string.rep['<5.1']          =
-'返回 `n` 个字符串 `s` 连在一起的字符串。 如果 `n` 不是正数则返回空串。'
 string.rep['>5.2']          =
 '返回 `n` 个字符串 `s` 以字符串 `sep` 为分割符连在一起的字符串。 默认的 `sep` 值为空字符串（即没有分割符）。 如果 `n` 不是正数则返回空串。'
+string.rep['<5.1']          =
+'返回 `n` 个字符串 `s` 连在一起的字符串。 如果 `n` 不是正数则返回空串。'
 string.reverse              =
 '返回字符串 s 的翻转串。'
 string.sub                  =
@@ -684,36 +684,6 @@ a1[f],···,a1[e]
 return a2
 ```
 ]]
-string.rep['>5.2']          = -- TODO: need translate!
-'Returns a string that is the concatenation of `n` copies of the string `s` separated by the string `sep`.'
-string.rep['<5.1']          = -- TODO: need translate!
-'Returns a string that is the concatenation of `n` copies of the string `s` .'
-string.reverse              = -- TODO: need translate!
-'Returns a string that is the string `s` reversed.'
-string.sub                  = -- TODO: need translate!
-'Returns the substring of the string that starts at `i` and continues until `j`.'
-string.unpack               = -- TODO: need translate!
-'Returns the values packed in string according to the format string `fmt` (see §6.4.2) .'
-string.upper                = -- TODO: need translate!
-'Returns a copy of this string with all lowercase letters changed to uppercase.'
-
-table                       = -- TODO: need translate!
-''
-table.concat                = -- TODO: need translate!
-'Given a list where all elements are strings or numbers, returns the string `list[i]..sep..list[i+1] ··· sep..list[j]`.'
-table.insert                = -- TODO: need translate!
-'Inserts element `value` at position `pos` in `list`.'
-table.maxn                  = -- TODO: need translate!
-'Returns the largest positive numerical index of the given table, or zero if the table has no positive numerical indices.'
-table.move                  = -- TODO: need translate!
-[[
-Moves elements from table `a1` to table `a2`.
-```lua
-a2[t],··· =
-a1[f],···,a1[e]
-return a2
-```
-]]
 table.pack                  =
 '返回用所有参数以键 `1`,`2`, 等填充的新表， 并将 `"n"` 这个域设为参数的总数。'
 table.remove                =
