@@ -85,7 +85,10 @@ function m.warn(...)
 end
 
 function m.error(...)
-    return pushLog('error', ...)
+    -- Don't use tail calls,
+    -- Otherwise, the count of `debug.getinfo` will be wrong
+    local msg = pushLog('error', ...)
+    return msg
 end
 
 function m.raw(thd, level, msg, source, currentline, clock)
@@ -124,7 +127,7 @@ function m.raw(thd, level, msg, source, currentline, clock)
         end
     end
 
-    if log.print then
+    if m.print then
         print(buf)
     end
 
