@@ -4,19 +4,24 @@ local type         = type
 ---@class parser.object
 ---@field bindDocs              parser.object[]
 ---@field bindGroup             parser.object[]
----@field bindSources           parser.object[]
+---@field bindSource            parser.object
 ---@field value                 parser.object
 ---@field parent                parser.object
 ---@field type                  string
 ---@field special               string
 ---@field tag                   string
----@field args                  parser.object[]
+---@field args                  { [integer]: parser.object, start: integer, finish: integer }
 ---@field locals                parser.object[]
----@field returns               parser.object[]
+---@field returns?              parser.object[]
+---@field breaks?               parser.object[]
+---@field exps                  parser.object[]
+---@field keys                  parser.object
 ---@field uri                   uri
 ---@field start                 integer
 ---@field finish                integer
+---@field range                 integer
 ---@field effect                integer
+---@field bstart                integer
 ---@field attrs                 string[]
 ---@field specials              parser.object[]
 ---@field labels                parser.object[]
@@ -41,6 +46,7 @@ local type         = type
 ---@field exp                   parser.object
 ---@field alias                 parser.object
 ---@field class                 parser.object
+---@field enum                  parser.object
 ---@field vararg                parser.object
 ---@field param                 parser.object
 ---@field overload              parser.object
@@ -48,7 +54,9 @@ local type         = type
 ---@field upvalues              table<string, string[]>
 ---@field ref                   parser.object[]
 ---@field returnIndex           integer
----@field docs                  parser.object[]
+---@field assignIndex           integer
+---@field docIndex              integer
+---@field docs                  parser.object
 ---@field state                 table
 ---@field comment               table
 ---@field optional              boolean
@@ -56,7 +64,19 @@ local type         = type
 ---@field init                  parser.object
 ---@field step                  parser.object
 ---@field redundant             { max: integer, passed: integer }
----@field _root                 parser.object
+---@field filter                parser.object
+---@field loc                   parser.object
+---@field keyword               integer[]
+---@field casts                 parser.object[]
+---@field mode?                 '+' | '-'
+---@field hasGoTo?              true
+---@field hasReturn?            true
+---@field hasBreak?             true
+---@field hasExit?              true
+---@field [integer]             parser.object|any
+---@field package _root         parser.object
+---@field package _eachCache?   parser.object[]
+---@field package _isGlobal?    boolean
 
 ---@class guide
 ---@field debugMode boolean
@@ -64,6 +84,10 @@ local m = {}
 
 m.ANY = {"<ANY>"}
 
+m.notNamePattern  = '[^%w_\x80-\xff]'
+m.namePattern     = '[%a_\x80-\xff][%w_\x80-\xff]*'
+m.namePatternFull = '^' .. m.namePattern .. '$'
+
 local blockTypes = {
     ['while']       = true,
     ['in']          = true,
@@ -71,6 +95,17 @@ local blockTypes = {
     ['repeat']      = true,
     ['do']          = true,
     ['function']    = true,
+    ['if']          = true,
+    ['ifblock']     = true,
+    ['elseblock']   = true,
+    ['elseifblock'] = true,
+    ['main']        = true,
+}
+
+local topBlockTypes = {
+    ['while']       = true,
+    ['function']    = true,
+    ['if']          = true,
     ['ifblock']     = true,
     ['elseblock']   = true,
     ['elseifblock'] = true,
@@ -91,39 +126,40 @@ local childMap = {
     ['while']       = {'filter', '#'},
     ['in']          = {'keys', 'exps', '#'},
     ['loop']        = {'loc', 'init', 'max', 'step', '#'},
+    ['do']          = {'#'},
     ['if']          = {'#'},
     ['ifblock']     = {'filter', '#'},
     ['elseifblock'] = {'filter', '#'},
     ['elseblock']   = {'#'},
     ['setfield']    = {'node', 'field', 'value'},
-    ['setglobal']   = {'value'},
-    ['local']       = {'attrs', 'value'},
-    ['setlocal']    = {'value'},
-    ['return']      = {'#'},
-    ['do']          = {'#'},
-    ['select']      = {'vararg'},
-    ['table']       = {'#'},
-    ['tableindex']  = {'index', 'value'},
-    ['tablefield']  = {'field', 'value'},
-    ['tableexp']    = {'value'},
-    ['function']    = {'args', '#'},
-    ['funcargs']    = {'#'},
+    ['getfield']    = {'node', 'field'},
     ['setmethod']   = {'node', 'method', 'value'},
     ['getmethod']   = {'node', 'method'},
     ['setindex']    = {'node', 'index', 'value'},
     ['getindex']    = {'node', 'index'},
+    ['tableindex']  = {'index', 'value'},
+    ['tablefield']  = {'field', 'value'},
+    ['tableexp']    = {'value'},
+    ['setglobal']   = {'value'},
+    ['local']       = {'attrs', 'value'},
+    ['setlocal']    = {'value'},
+    ['return']      = {'#'},
+    ['select']      = {'vararg'},
+    ['table']       = {'#'},
+    ['function']    = {'args', '#'},
+    ['funcargs']    = {'#'},
     ['paren']       = {'exp'},
     ['call']        = {'node', 'args'},
     ['callargs']    = {'#'},
-    ['getfield']    = {'node', 'field'},
     ['list']        = {'#'},
     ['binary']      = {1, 2},
     ['unary']       = {1},
 
     ['doc']                = {'#'},
-    ['doc.class']          = {'class', '#extends', '#signs', 'comment'},
+    ['doc.class']          = {'class', '#extends', '#signs', 'docAttr', 'comment'},
     ['doc.type']           = {'#types', 'name', 'comment'},
     ['doc.alias']          = {'alias', 'extends', 'comment'},
+    ['doc.enum']           = {'enum', 'extends', 'comment', 'docAttr'},
     ['doc.param']          = {'param', 'extends', 'comment'},
     ['doc.return']         = {'#returns', 'comment'},
     ['doc.field']          = {'field', 'extends', 'comment'},
@@ -138,9 +174,15 @@ local childMap = {
     ['doc.type.field']     = {'name', 'extends'},
     ['doc.type.sign']      = {'node', '#signs'},
     ['doc.overload']       = {'overload', 'comment'},
-    ['doc.see']            = {'name', 'field'},
+    ['doc.see']            = {'name', 'comment'},
     ['doc.version']        = {'#versions'},
     ['doc.diagnostic']     = {'#names'},
+    ['doc.as']             = {'as'},
+    ['doc.cast']           = {'name', '#casts'},
+    ['doc.cast.block']     = {'extends'},
+    ['doc.operator']       = {'op', 'exp', 'extends'},
+    ['doc.meta']           = {'name'},
+    ['doc.attr']           = {'#names'},
 }
 
 ---@type table<string, fun(obj: parser.object, list: parser.object[])>
@@ -181,6 +223,43 @@ end
     return f
 end})
 
+local eachChildMap = setmetatable({}, {__index = function (self, name)
+    local defs = childMap[name]
+    if not defs then
+        self[name] = false
+        return false
+    end
+    local text = {}
+    text[#text+1] = 'local obj, callback = ...'
+    for _, def in ipairs(defs) do
+        if def == '#' then
+            text[#text+1] = [[
+for i = 1, #obj do
+    callback(obj[i])
+end
+]]
+        elseif type(def) == 'string' and def:sub(1, 1) == '#' then
+            local key = def:sub(2)
+            text[#text+1] = ([[
+local childs = obj.%s
+if childs then
+    for i = 1, #childs do
+        callback(childs[i])
+    end
+end
+]]):format(key)
+        elseif type(def) == 'string' then
+            text[#text+1] = ('callback(obj.%s)'):format(def)
+        else
+            text[#text+1] = ('callback(obj[%q])'):format(def)
+        end
+    end
+    local buf = table.concat(text, '\n')
+    local f = load(buf, buf, 't')
+    self[name] = f
+    return f
+end})
+
 m.actionMap = {
     ['main']        = {'#'},
     ['repeat']      = {'#'},
@@ -196,34 +275,8 @@ m.actionMap = {
     ['funcargs']    = {'#'},
 }
 
-local inf          = 1 / 0
-local nan          = 0 / 0
-
-local function isInteger(n)
-    if math.type then
-        return math.type(n) == 'integer'
-    else
-        return type(n) == 'number' and n % 1 == 0
-    end
-end
-
-local function formatNumber(n)
-    if n == inf
-    or n == -inf
-    or n == nan
-    or n ~= n then -- IEEE 标准中，NAN 不等于自己。但是某些实现中没有遵守这个规则
-        return ('%q'):format(n)
-    end
-    if isInteger(n) then
-        return tostring(n)
-    end
-    local str = ('%.10f'):format(n)
-    str = str:gsub('%.?0*$', '')
-    return str
-end
-
 --- 是否是字面量
----@param obj parser.object
+---@param obj table
 ---@return boolean
 function m.isLiteral(obj)
     local tp = obj.type
@@ -239,20 +292,15 @@ function m.isLiteral(obj)
         or tp == 'doc.type.string'
         or tp == 'doc.type.integer'
         or tp == 'doc.type.boolean'
+        or tp == 'doc.type.code'
+        or tp == 'doc.type.array'
 end
 
 --- 获取字面量
----@param obj parser.object
+---@param obj table
 ---@return any
 function m.getLiteral(obj)
-    local tp = obj.type
-    if     tp == 'boolean' then
-        return obj[1]
-    elseif tp == 'string' then
-        return obj[1]
-    elseif tp == 'number' then
-        return obj[1]
-    elseif tp == 'integer' then
+    if m.isLiteral(obj) then
         return obj[1]
     end
     return nil
@@ -260,7 +308,7 @@ end
 
 --- 寻找父函数
 ---@param obj parser.object
----@return parser.object
+---@return parser.object?
 function m.getParentFunction(obj)
     for _ = 1, 10000 do
         obj = obj.parent
@@ -277,7 +325,7 @@ end
 
 --- 寻找所在区块
 ---@param obj parser.object
----@return parser.object
+---@return parser.object?
 function m.getBlock(obj)
     for _ = 1, 10000 do
         if not obj then
@@ -306,7 +354,7 @@ end
 
 --- 寻找所在父区块
 ---@param obj parser.object
----@return parser.object
+---@return parser.object?
 function m.getParentBlock(obj)
     for _ = 1, 10000 do
         obj = obj.parent
@@ -323,7 +371,7 @@ end
 
 --- 寻找所在可break的父区块
 ---@param obj parser.object
----@return parser.object
+---@return parser.object?
 function m.getBreakBlock(obj)
     for _ = 1, 10000 do
         obj = obj.parent
@@ -360,7 +408,7 @@ end
 
 --- 寻找所在父类型
 ---@param obj parser.object
----@return parser.object
+---@return parser.object?
 function m.getParentType(obj, want)
     for _ = 1, 10000 do
         obj = obj.parent
@@ -393,14 +441,14 @@ function m.getRoot(obj)
         end
         local parent = obj.parent
         if not parent then
-            return nil
+            error('Can not find out root:' .. tostring(obj.type))
         end
         obj = parent
     end
     error('guide.getRoot overstack')
 end
 
----@param obj parser.object
+---@param obj parser.object | { uri: uri }
 ---@return uri
 function m.getUri(obj)
     if obj.uri then
@@ -413,6 +461,7 @@ function m.getUri(obj)
     return ''
 end
 
+---@return parser.object?
 function m.getENV(source, start)
     if not start then
         start = 1
@@ -421,63 +470,57 @@ function m.getENV(source, start)
         or m.getLocal(source, '@fenv', start)
 end
 
---- 寻找函数的不定参数，返回不定参在第几个参数上，以及该参数对象。
---- 如果函数是主函数，则返回`0, nil`。
----@return table
----@return integer
-function m.getFunctionVarArgs(func)
-    if func.type == 'main' then
-        return 0, nil
-    end
-    if func.type ~= 'function' then
-        return nil, nil
-    end
-    local args = func.args
-    if not args then
-        return nil, nil
-    end
-    for i = 1, #args do
-        local arg = args[i]
-        if arg.type == '...' then
-            return i, arg
-        end
-    end
-    return nil, nil
-end
-
 --- 获取指定区块中可见的局部变量
----@param block table
----@param name string {comment = '变量名'}
----@param pos integer {comment = '可见位置'}
-function m.getLocal(block, name, pos)
-    block = m.getBlock(block)
+---@param source parser.object
+---@param name string # 变量名
+---@param pos integer # 可见位置
+---@return parser.object?
+function m.getLocal(source, name, pos)
+    local block = source
+    -- find nearest source
     for _ = 1, 10000 do
         if not block then
             return nil
         end
-        local locals = block.locals
-        local res
-        if not locals then
-            goto CONTINUE
+        if block.type == 'main' then
+            break
         end
-        for i = 1, #locals do
-            local loc = locals[i]
-            if loc.effect > pos then
-                break
-            end
-            if loc[1] == name then
-                if not res or res.effect < loc.effect then
-                    res = loc
+        if  block.start <= pos
+        and block.finish >= pos
+        and blockTypes[block.type] then
+            break
+        end
+        block = block.parent
+    end
+
+    m.eachSourceContain(block, pos, function (src)
+        if  blockTypes[src.type]
+        and (src.finish - src.start) < (block.finish - src.start) then
+            block = src
+        end
+    end)
+
+    for _ = 1, 10000 do
+        if not block then
+            break
+        end
+        local res
+        if block.locals then
+            for _, loc in ipairs(block.locals) do
+                if  loc[1] == name
+                and loc.effect <= pos then
+                    if not res or res.effect < loc.effect then
+                        res = loc
+                    end
                 end
             end
         end
         if res then
-            return res, res
+            return res
         end
-        ::CONTINUE::
-        block = m.getParentBlock(block)
+        block = block.parent
     end
-    error('guide.getLocal overstack')
+    return nil
 end
 
 --- 获取指定区块中所有的可见局部变量名称
@@ -499,25 +542,25 @@ function m.getVisibleLocals(block, pos)
 end
 
 --- 获取指定区块中可见的标签
----@param block table
----@param name string {comment = '标签名'}
+---@param block parser.object
+---@param name string
 function m.getLabel(block, name)
-    block = m.getBlock(block)
+    local current = m.getBlock(block)
     for _ = 1, 10000 do
-        if not block then
+        if not current then
             return nil
         end
-        local labels = block.labels
+        local labels = current.labels
         if labels then
             local label = labels[name]
             if label then
                 return label
             end
         end
-        if block.type == 'function' then
+        if current.type == 'function' then
             return nil
         end
-        block = m.getParentBlock(block)
+        current = m.getParentBlock(current)
     end
     error('guide.getLocal overstack')
 end
@@ -602,6 +645,9 @@ local function addChilds(list, obj)
 end
 
 --- 遍历所有包含position的source
+---@param ast parser.object
+---@param position integer
+---@param callback fun(src: parser.object): any
 function m.eachSourceContain(ast, position, callback)
     local list = { ast }
     local mark = {}
@@ -677,7 +723,7 @@ end
 --- 遍历所有指定类型的source
 ---@param ast parser.object
 ---@param type string
----@param callback fun(src: parser.object)
+---@param callback fun(src: parser.object): any
 ---@return any
 function m.eachSourceType(ast, type, callback)
     local cache = getSourceTypeCache(ast)
@@ -710,7 +756,7 @@ end
 
 --- 遍历所有的source
 ---@param ast parser.object
----@param callback fun(src: parser.object)
+---@param callback fun(src: parser.object): boolean?
 function m.eachSource(ast, callback)
     local cache = ast._eachCache
     if not cache then
@@ -738,7 +784,20 @@ function m.eachSource(ast, callback)
     end
 end
 
+---@param source   parser.object
+---@param callback fun(src: parser.object)
+function m.eachChild(source, callback)
+    local f = eachChildMap[source.type]
+    if not f then
+        return
+    end
+    f(source, callback)
+end
+
 --- 获取指定的 special
+---@param ast parser.object
+---@param name string
+---@param callback fun(src: parser.object)
 function m.eachSpecialOf(ast, name, callback)
     local root = m.getRoot(ast)
     local state = root.state
@@ -771,12 +830,24 @@ end
 ---@param col integer
 ---@return integer
 function m.positionOf(row, col)
-    return row * 10000 + col
+    return row * 10000 + math.min(col, 10000 - 1)
 end
 
 function m.positionToOffsetByLines(lines, position)
     local row, col = m.rowColOf(position)
-    return (lines[row] or 1) + col - 1
+    if row < 0 then
+        return 0
+    end
+    if row > #lines then
+        return lines.size
+    end
+    local offset = lines[row] + col - 1
+    if lines[row + 1] and offset >= lines[row + 1] then
+        return lines[row + 1] - 1
+    elseif offset > lines.size then
+        return lines.size
+    end
+    return offset
 end
 
 --- 返回全文光标位置
@@ -786,6 +857,8 @@ function m.positionToOffset(state, position)
     return m.positionToOffsetByLines(state.lines, position)
 end
 
+---@param lines integer[]
+---@param offset integer
 function m.offsetToPositionByLines(lines, offset)
     local left  = 0
     local right = #lines
@@ -829,7 +902,7 @@ function m.getLineRange(state, row)
     return 0
 end
 
-local isSetMap = {
+local assignTypeMap = {
     ['setglobal']         = true,
     ['local']             = true,
     ['self']              = true,
@@ -839,19 +912,21 @@ local isSetMap = {
     ['setindex']          = true,
     ['tablefield']        = true,
     ['tableindex']        = true,
-    ['tableexp']          = true,
     ['label']             = true,
     ['doc.class']         = true,
     ['doc.alias']         = true,
+    ['doc.enum']          = true,
     ['doc.field']         = true,
     ['doc.class.name']    = true,
     ['doc.alias.name']    = true,
+    ['doc.enum.name']     = true,
     ['doc.field.name']    = true,
     ['doc.type.field']    = true,
+    ['doc.type.array']    = true,
 }
-function m.isSet(source)
+function m.isAssign(source)
     local tp = source.type
-    if isSetMap[tp] then
+    if assignTypeMap[tp] then
         return true
     end
     if tp == 'call' then
@@ -863,7 +938,7 @@ function m.isSet(source)
     return false
 end
 
-local isGetMap = {
+local getTypeMap = {
     ['getglobal'] = true,
     ['getlocal']  = true,
     ['getfield']  = true,
@@ -872,7 +947,7 @@ local isGetMap = {
 }
 function m.isGet(source)
     local tp = source.type
-    if isGetMap[tp] then
+    if getTypeMap[tp] then
         return true
     end
     if tp == 'call' then
@@ -899,29 +974,18 @@ function m.getKeyNameOfLiteral(obj)
     if tp == 'field'
     or     tp == 'method' then
         return obj[1]
-    elseif tp == 'string' then
-        local s = obj[1]
-        if s then
-            return s
-        end
-    elseif tp == 'number' then
-        local n = obj[1]
-        if n then
-            return obj[1]
-        end
-    elseif tp == 'integer' then
-        local n = obj[1]
-        if n then
-            return obj[1]
-        end
-    elseif tp == 'boolean' then
-        local b = obj[1]
-        if b then
-            return b
-        end
+    elseif tp == 'string'
+    or     tp == 'number'
+    or     tp == 'integer'
+    or     tp == 'boolean'
+    or     tp == 'doc.type.integer'
+    or     tp == 'doc.type.string'
+    or     tp == 'doc.type.boolean' then
+        return obj[1]
     end
 end
 
+---@return string?
 function m.getKeyName(obj)
     if not obj then
         return nil
@@ -953,19 +1017,25 @@ function m.getKeyName(obj)
     elseif tp == 'tableexp' then
         return obj.tindex
     elseif tp == 'field'
-    or     tp == 'method'
-    or     tp == 'doc.see.field' then
+    or     tp == 'method' then
         return obj[1]
     elseif tp == 'doc.class' then
         return obj.class[1]
     elseif tp == 'doc.alias' then
         return obj.alias[1]
+    elseif tp == 'doc.enum' then
+        return obj.enum[1]
     elseif tp == 'doc.field' then
         return obj.field[1]
-    elseif tp == 'doc.field.name' then
+    elseif tp == 'doc.field.name'
+    or     tp == 'doc.type.name'
+    or     tp == 'doc.class.name'
+    or     tp == 'doc.alias.name'
+    or     tp == 'doc.enum.name'
+    or     tp == 'doc.extends.name' then
         return obj[1]
     elseif tp == 'doc.type.field' then
-        return obj.name[1]
+        return m.getKeyName(obj.name)
     end
     return m.getKeyNameOfLiteral(obj)
 end
@@ -1016,13 +1086,14 @@ function m.getKeyType(obj)
     elseif tp == 'tableexp' then
         return 'integer'
     elseif tp == 'field'
-    or     tp == 'method'
-    or     tp == 'doc.see.field' then
+    or     tp == 'method' then
         return 'string'
     elseif tp == 'doc.class' then
         return 'string'
     elseif tp == 'doc.alias' then
         return 'string'
+    elseif tp == 'doc.enum' then
+        return 'string'
     elseif tp == 'doc.field' then
         return type(obj.field[1])
     elseif tp == 'doc.type.field' then
@@ -1034,95 +1105,6 @@ function m.getKeyType(obj)
     return m.getKeyTypeOfLiteral(obj)
 end
 
---- 测试 a 到 b 的路径（不经过函数，不考虑 goto），
---- 每个路径是一个 block 。
----
---- 如果 a 在 b 的前面，返回 `"before"` 加上 2个`list<block>`
----
---- 如果 a 在 b 的后面，返回 `"after"` 加上 2个`list<block>`
----
---- 否则返回 `false`
----
---- 返回的2个 `list` 分别为基准block到达 a 与 b 的路径。
----@param a table
----@param b table
----@return string|boolean mode
----@return table pathA?
----@return table pathB?
-function m.getPath(a, b, sameFunction)
-    --- 首先测试双方在同一个函数内
-    if sameFunction and m.getParentFunction(a) ~= m.getParentFunction(b) then
-        return false
-    end
-    local mode
-    local objA
-    local objB
-    if a.finish < b.start then
-        mode = 'before'
-        objA = a
-        objB = b
-    elseif a.start > b.finish then
-        mode = 'after'
-        objA = b
-        objB = a
-    else
-        return 'equal', {}, {}
-    end
-    local pathA = {}
-    local pathB = {}
-    for _ = 1, 1000 do
-        objA = m.getParentBlock(objA)
-        pathA[#pathA+1] = objA
-        if (not sameFunction and objA.type == 'function') or objA.type == 'main' then
-            break
-        end
-    end
-    for _ = 1, 1000 do
-        objB = m.getParentBlock(objB)
-        pathB[#pathB+1] = objB
-        if (not sameFunction and objA.type == 'function') or objB.type == 'main' then
-            break
-        end
-    end
-    -- pathA: {1, 2, 3, 4, 5}
-    -- pathB: {5, 6, 2, 3}
-    local top = #pathB
-    local start
-    for i = #pathA, 1, -1 do
-        local currentBlock = pathA[i]
-        if currentBlock == pathB[top] then
-            start = i
-            break
-        end
-    end
-    if not start then
-        return nil
-    end
-    -- pathA: {   1, 2, 3}
-    -- pathB: {5, 6, 2, 3}
-    local extra = 0
-    local align = top - start
-    for i = start, 1, -1 do
-        local currentA = pathA[i]
-        local currentB = pathB[i+align]
-        if currentA ~= currentB then
-            extra = i
-            break
-        end
-    end
-    -- pathA: {1}
-    local resultA = {}
-    for i = extra, 1, -1 do
-        resultA[#resultA+1] = pathA[i]
-    end
-    -- pathB: {5, 6}
-    local resultB = {}
-    for i = extra + align, 1, -1 do
-        resultB[#resultB+1] = pathB[i]
-    end
-    return mode, resultA, resultB
-end
-
 ---是否是全局变量（包括 _G.XXX 形式）
 ---@param source parser.object
 ---@return boolean
@@ -1189,6 +1171,15 @@ function m.isInString(ast, position)
     end)
 end
 
+function m.isInComment(ast, offset)
+    for _, com in ipairs(ast.state.comms) do
+        if offset >= com.start and offset <= com.finish then
+            return true
+        end
+    end
+    return false
+end
+
 function m.isOOP(source)
     if source.type == 'setmethod'
     or source.type == 'getmethod' then
@@ -1209,6 +1200,7 @@ local basicTypeMap = {
     ['false']    = true,
     ['nil']      = true,
     ['boolean']  = true,
+    ['integer']  = true,
     ['number']   = true,
     ['string']   = true,
     ['table']    = true,
@@ -1223,4 +1215,81 @@ function m.isBasicType(str)
     return basicTypeMap[str] == true
 end
 
+---@param source parser.object
+---@return boolean
+function m.isBlockType(source)
+    return blockTypes[source.type] == true
+end
+
+---@param source parser.object
+---@return parser.object?
+function m.getSelfNode(source)
+    if source.type == 'getlocal'
+    or source.type == 'setlocal' then
+        source = source.node
+    end
+    if source.type ~= 'self' then
+        return nil
+    end
+    local args = source.parent
+    if args.type == 'callargs' then
+        local call = args.parent
+        if call.type ~= 'call' then
+            return nil
+        end
+        local getmethod = call.node
+        if getmethod.type ~= 'getmethod' then
+            return nil
+        end
+        return getmethod.node
+    end
+    if args.type == 'funcargs' then
+        return m.getFunctionSelfNode(args.parent)
+    end
+    return nil
+end
+
+---@param func parser.object
+---@return parser.object?
+function m.getFunctionSelfNode(func)
+    if func.type ~= 'function' then
+        return nil
+    end
+    local parent = func.parent
+    if parent.type == 'setmethod'
+    or parent.type == 'setfield' then
+        return parent.node
+    end
+    return nil
+end
+
+---@param source parser.object
+---@return parser.object?
+function m.getTopBlock(source)
+    for _ = 1, 1000 do
+        local block = source.parent
+        if not block then
+            return nil
+        end
+        if topBlockTypes[block.type] then
+            return block
+        end
+        source = block
+    end
+    return nil
+end
+
+---@param source parser.object
+---@return boolean
+function m.isParam(source)
+    if  source.type ~= 'local'
+    and source.type ~= 'self' then
+        return false
+    end
+    if source.parent.type ~= 'funcargs' then
+        return false
+    end
+    return true
+end
+
 return m
