@@ -91,7 +91,7 @@ function mt:resolve(uri, args)
                 if n.type == 'global' and n.cate == 'type' then
                     -- ---@field [integer]: number -> T[]
                     ---@cast n vm.global
-                    vm.getClassFields(uri, n, vm.declareGlobal('type', 'integer'), false, function (field)
+                    vm.getClassFields(uri, n, vm.declareGlobal('type', 'integer'), function (field)
                         resolve(object.node, vm.compileNode(field.extends))
                     end)
                 end
@@ -142,13 +142,15 @@ function mt:resolve(uri, args)
         end
         if object.type == 'doc.type.function' then
             for i, arg in ipairs(object.args) do
-                for n in node:eachObject() do
-                    if n.type == 'function'
-                    or n.type == 'doc.type.function' then
-                        ---@cast n parser.object
-                        local farg = n.args and n.args[i]
-                        if farg then
-                            resolve(arg.extends, vm.compileNode(farg))
+                if arg.extends then
+                    for n in node:eachObject() do
+                        if n.type == 'function'
+                        or n.type == 'doc.type.function' then
+                            ---@cast n parser.object
+                            local farg = n.args and n.args[i]
+                            if farg then
+                                resolve(arg.extends, vm.compileNode(farg))
+                            end
                         end
                     end
                 end
@@ -193,7 +195,11 @@ function mt:resolve(uri, args)
                     goto CONTINUE
                 end
             end
-            local view = vm.viewObject(obj, uri)
+            if obj.type == 'variable'
+            or obj.type == 'local' then
+                goto CONTINUE
+            end
+            local view = vm.getInfer(obj):view(uri)
             if view then
                 knownTypes[view] = true
             end
@@ -219,7 +225,7 @@ function mt:resolve(uri, args)
                     goto CONTINUE
                 end
             end
-            local view = vm.viewObject(n, uri)
+            local view = vm.getInfer(n):view(uri)
             if knownTypes[view] then
                 goto CONTINUE
             end
@@ -250,7 +256,7 @@ function mt:resolve(uri, args)
         local argNode = vm.compileNode(arg)
         local knownTypes, genericNames = getSignInfo(sign)
         if not isAllResolved(genericNames) then
-            local newArgNode = buildArgNode(argNode,sign, knownTypes)
+            local newArgNode = buildArgNode(argNode, sign, knownTypes)
             resolve(sign, newArgNode)
         end
     end
